#pragma once
#include "audio_device.h"

#include "portaudio.h"
#include "audio/misc/circular_audio_buffer.h"

class port_audio_device : public audio_device {
public:
    ~port_audio_device() override;
    bool init() override;
    void destroy() override;

    void stop();

    void start();
    void open_stream(PaDeviceIndex input_device, PaDeviceIndex output_device, double in_sample_rate, uint32_t in_buffer_size);

    int stream_callback(float** input, float** output, unsigned long frame_count, const PaStreamCallbackTimeInfo* time_info, PaStreamCallbackFlags status_flags);
protected:
    bool on_set_buffer_size(uint32_t in_buffer_size) override;
    bool on_set_sample_rate(double in_sample_rate) override;
private:

#pragma region render_thread
    void start_render_thread();
    void stop_render_thread();
    void render_thread();
    std::thread render_thread_;
    circular_audio_buffer<float> render_buffer_;
    uint32_t render_buffer_size_ = 2048;
    std::atomic_bool render_thread_running_ = false;
#pragma endregion

    PaStreamParameters input_params_ = {};
    PaStreamParameters output_params_ = {};
    PaStream* stream_ = nullptr;
};
